#include "project.h"
#include "stdio.h"
#include "tty_utils.h"

#define IR_ST_IDLE       0
#define IR_ST_START_BIT  1
#define IR_ST_TOGGLE     2
#define IR_ST_ADDRESS    3
#define IR_ST_COMMAND    4
#define IR_ST_DATA_READY 5

#define IR_ADDRESS_WIDTH 5
#define IR_COMMAND_WIDTH 6

#define MY_IR_ADDRESS    3
#define MY_IR_COMMAND    7

volatile int ir_status = IR_ST_IDLE ;
volatile int ir_address_bit = 0 ; /* 5bit */
volatile int ir_command_bit = 0 ; /* 6bit */
volatile int ir_toggle      = 0 ;
volatile int ir_address     = 0 ;
volatile int ir_command     = 0 ;
volatile int capsense_running = 0 ;

CY_ISR_PROTO(timer_isr) ;

CY_ISR(ir_isr)
{
    isr_ir_ClearPending() ;
    
    if (ir_status == IR_ST_IDLE) {
        if (capsense_running) {
            CapSense_Stop() ;
            capsense_running = 0 ;
        }
        ir_status = IR_ST_START_BIT ;
        isr_ir_Disable() ;
        isr_ir_ClearPending() ;
        isr_bit_ClearPending() ;
        isr_bit_StartEx(timer_isr) ;
        bit_timer_ClearInterrupt(bit_timer_INTR_MASK_CC_MATCH) ;
        bit_timer_WriteCounter(0) ;
        bit_timer_Enable() ;
    }
}

CY_ISR(timer_isr)
{
    int ir_in ;
    
    bit_timer_ClearInterrupt(bit_timer_INTR_MASK_CC_MATCH) ;
    ir_in = IR_input_Read() ;
    
    switch(ir_status) {
    case IR_ST_IDLE: // should not come here
        break ;
    case IR_ST_START_BIT:
        if (ir_in == 1) {
            ir_status = IR_ST_TOGGLE ;
        } else {
            ir_status = IR_ST_IDLE ;
        }
        break ;
    case IR_ST_TOGGLE:
        ir_toggle = ir_in ;
        ir_status = IR_ST_ADDRESS ;
        break ;
    case IR_ST_ADDRESS:
        ir_address <<= 1 ;
        ir_address |= ir_in ;
        if (++ir_address_bit >= IR_ADDRESS_WIDTH) {
            ir_status = IR_ST_COMMAND ;
        }
        break ;
    case IR_ST_COMMAND:
        ir_command <<= 1 ;
        ir_command |= ir_in ;
        if (++ir_command_bit >= IR_COMMAND_WIDTH) {
            ir_status = IR_ST_DATA_READY ;
            isr_bit_Disable() ;
            isr_bit_ClearPending() ;
            bit_timer_Stop() ;
        }
        break ;
    case IR_ST_DATA_READY: // should not come here
    default:               // should not come here
        break ;
    }
}

void clear_data(void)
{
    ir_toggle = 0 ;
    ir_address = 0 ;
    ir_address_bit = 0 ;
    ir_command = 0 ;
    ir_command_bit = 0 ;
}

void init_hardware(void)
{
    CyGlobalIntEnable; /* Enable global interrupts. */
    
    tty_init() ;
    
    splash("RC5 Test") ;
    
    isr_ir_ClearPending() ;
    isr_ir_StartEx(ir_isr) ;
    
    isr_bit_ClearPending() ;
    isr_bit_StartEx(timer_isr) ;
    
    bit_timer_Init() ;
    
    capsense_running = 1 ;
    CapSense_Start() ;
    CapSense_ScanAllWidgets() ;
}

int main(void)
{   
    uint8_t status, prev_status ;
    int my_ir_address = MY_IR_ADDRESS ;
    int my_ir_command = MY_IR_COMMAND ;
    int toggle_led_flag = 0 ;
    
    init_hardware() ;

    for(;;)
    {
        if (ir_status == IR_ST_DATA_READY) {
            snprintf(str, STR_BUF_LEN, "toggle: %d address %d: command %d\n\r", 
                ir_toggle, ir_address, ir_command) ;
            print(str) ;
            if ((ir_address == MY_IR_ADDRESS) && (ir_command == MY_IR_COMMAND)) {
                print("IR Command Hit!\n\r") ;
                toggle_led_flag = 1 ;
            }

            if (capsense_running == 0) {
                capsense_running = 1 ;
                CapSense_Start() ;
                CapSense_ScanAllWidgets() ;
            }
            
            clear_data() ;
            ir_status = IR_ST_IDLE ;
            isr_ir_ClearPending() ;
            isr_ir_StartEx(ir_isr) ;
            // isr_ir_Enable() ;
        }
        if (capsense_running && (CapSense_NOT_BUSY == CapSense_IsBusy())) {
            CapSense_ProcessAllWidgets() ;
            status = CapSense_IsWidgetActive(CapSense_BUTTON08_WDGT_ID) ;
            if (status && !prev_status) { /* CapSense Activated Edge */
                print("CapSense Button08 Touched!\n\r") ;
                toggle_led_flag = 1 ;
            }
            prev_status = status ;
            CapSense_ScanAllWidgets() ;
        }
        if (toggle_led_flag) {
            toggle_led_flag = 0 ;
            print("LED: ") ;
            if (LED_Read()) {
                LED_Write(0) ;
                print("ON\n\r") ;
            } else {
                LED_Write(1) ;
                print("OFF\n\r") ;
            }
        }
    }
}

